#!/usr/bin/env python3

import numpy as np
import sys
from os import listdir, stat, path

if (len(sys.argv) < 2):
    print("[RESULT-PATH] [TOP-DIRECTORY]",
           file=sys.stderr)
    sys.exit(1)

resultpath = sys.argv[1]
topdir = sys.argv[2]
process_name = (path.splitext(path.basename(topdir))[0])

x = []
m = []
s = []

rerun = []
for dir in listdir(topdir):
    if not path.isdir(path.join(topdir, dir)):
        continue
    file = f"{topdir}/{dir}/{process_name}.time"
    log = f"{process_name} {topdir}/{dir} {dir}"
    if not path.exists(file) or not stat(file).st_size > 0:
        rerun.append(log)
        continue
    n_task = int(path.dirname(file).split('/')[-1])
    data = np.genfromtxt (file)
    if not data.size > 1:
        rerun.append(log)
        continue
    x.append(n_task)
    m.append(np.mean (data[-3:]))
    s.append(np.std (data[-3:]))

if len(rerun) > 0:
    for log in rerun:
        print(log, file=sys.stderr)
    sys.exit(2)

index = np.argsort(x)
data = np.column_stack([np.array(x)[index], np.array(m)[index], np.array(s)[index]])
np.savetxt(f"{resultpath}/{topdir}.time", data,
            header="# Tasks Mean Variance - Runtime")

with open(f"{resultpath}/{topdir}.tex", 'w') as file:
    file.write('\n'.join(["{:d} & {:.1f} & {:.1f} \\\\".format(int(n), mean, std) for n, mean, std in data]))
