#!/usr/bin/env python3

from whizard import latex_processes, lhc_processes
import sys
from os import getcwd, path

import jinja2 as j2
import numpy as np
from uncertainties import ufloat
from uncertainties.umath import *

if (len(sys.argv) < 1):
    print("You have to specify a filename!")

file = sys.argv[1]
filename = (path.splitext(path.basename(file))[0])

data = np.genfromtxt(file, dtype=None)
processes = []
for line in data:
    name = line[0].decode('unicode_escape')
    value = ufloat(line[1], line[2])
    processes += [
       	{
	    'name': name,
            'code': latex_processes[name]['code'],
	    'value': "{:S}".format(value),
            'compare': lhc_processes[name]
        }
    ] 

templateLoader = j2.FileSystemLoader(searchpath=getcwd())
templateEnv = j2.Environment(loader=templateLoader)
template_file = "{}-template.tex".format(filename)
result_file = "{}.tex".format(filename)
template = templateEnv.get_template(template_file)
with open(result_file, 'w') as outfile:
    outfile.write(template.render(
        processes=sorted(processes, key=lambda x: x['name'])))
